.allocate-layout:not(.allocate-mode) .allocate-mode-layout,
.allocate-layout.allocate-mode .allocate-columns-layout {
  display: none;
}

/* Allocate mode content container */
.allocate-layout .allocate-mode-content-scroller {
  overflow: auto;
  flex: 1;
}
.allocate-layout .allocate-mode-content-container {
  display: flex;
  flex-direction: row;
  gap: 10px;
  margin: 10px 0;
}
.allocate-layout .allocate-mode-content-container .allocate-table {
  display: table;
  border: 0;
  margin: 0;
  /* table-layout: fixed; */
  border-style: solid;
  border-spacing: 10px;
  border-collapse: separate;
}
.allocate-layout .allocate-mode-content-container .allocate-table .allocate-row {
  display: table-row;
}
.allocate-layout .allocate-mode-content-container .allocate-table .allocate-cell {
  --column-width: 250px;

  border: 0;
  display: table-cell;
  vertical-align: top;
  /* NOTE: Min and max width don't work for table cells
   * min-width: 250px;
   * max-width: 500px;
   */
  width: var(--column-width);
  max-width: var(--column-width);
  min-width: var(--column-width);
  overflow: hidden;
}
.allocate-layout .allocate-mode-content-container .allocate-table .allocate-cell > div {
  overflow: hidden;
  text-overflow: ellipsis;
  width: 100%;
  position: relative;
}
.allocate-layout .allocate-mode-content-container .allocate-table .production-group-cell {
  padding: 0;
}

.allocate-layout .allocate-mode-layout .production-header {
  padding: 5px 10px;
  background-color: var(--layout-theme-primary-color);
  color: #fff;
  border-radius: 4px;
}
.allocate-layout .allocate-mode-layout .production-group {
  /* padding: 5px 10px; */
  background-color: #eee;
  border-radius: 4px;
  overflow: hidden;
  display: flex;
  flex-direction: column;
  gap: 10px;
}
.allocate-layout .allocate-mode-layout .production-group > .header {
  padding: 5px 10px;
  background-color: #ccc;
}
.allocate-layout .allocate-mode-layout .group-item > .header > .title,
.allocate-layout .allocate-mode-layout .production-group > .header > .title {
  overflow: hidden;
  text-overflow: ellipsis;
  width: 100%;
  position: relative;
}
.allocate-layout .allocate-mode-layout .production-group .group-fraction {
  padding: 0 8px;
}
.allocate-layout .allocate-mode-layout .production-group .group-fraction input {
  width: 100%;
  margin: 0;
}
.allocate-layout .allocate-mode-layout .production-group .group-fraction input.error {
  border-color: var(--common-error-color);
  box-shadow: 0 0 4px var(--common-error-color);
}
.allocate-layout .allocate-mode-layout .production-group.last .group-fraction input {
  background-color: rgba(255 255 255 / 50%);
  opacity: 0.5;
  pointer-events: none;
}
.allocate-layout .allocate-mode-layout .production-group .group-items {
  padding: 0 10px;
  padding-bottom: 7px;
}
.allocate-layout .allocate-mode-layout .production-group .group-items .list {
  display: flex;
  flex-direction: column;
  gap: 5px;
  padding-top: 10px;
}
.allocate-layout .allocate-mode-layout .production-group .group-item .type {
  opacity: 0.5;
}

/* Collapsed/expanded group items */
.allocate-layout .allocate-mode-layout .production-group .action:not(.disabled) {
  transition: all var(--common-animation-time);
  /* opacity: 0.6; */
  cursor: pointer;
}
.allocate-layout .allocate-mode-layout .production-group .action:not(.disabled):hover {
  opacity: 1;
  /* text-decoration: underline; */
}
.allocate-layout .allocate-mode-layout .production-group .group-items:not(.empty) .shortcut.empty,
.allocate-layout .allocate-mode-layout .production-group .group-items.empty .shortcut.action,
.allocate-layout .allocate-mode-layout .production-group .group-items:not(.expanded) .list {
  display: none;
}
.allocate-layout .allocate-mode-layout .production-group .group-items .shortcut {
  display: flex;
  align-items: center;
  gap: 5px;
  /* justify-content: center; */
}
.allocate-layout .allocate-mode-layout .production-group .group-items .shortcut .label {
  opacity: 0.5;
}
.allocate-layout .allocate-mode-layout .production-group .group-items .shortcut .handler {
  transition: background-color var(--common-animation-time);
  color: var(--layout-theme-primary-color);
  margin-right: 2px;
  width: 24px;
  height: 24px;
  border-radius: 50%;
  display: flex;
  flex-shrink: 0;
  align-items: center;
  justify-content: center;
}
.allocate-layout .allocate-mode-layout .production-group .group-items .shortcut:hover .handler {
  background-color: var(--layout-theme-primary-color);
  color: #fff;
}
.allocate-layout .allocate-mode-layout .production-group .group-items .shortcut .handler .fa {
  transition: all var(--common-animation-time);
  font-size: 75%;
}
.allocate-layout
  .allocate-mode-layout
  .production-group
  .group-items.expanded
  .shortcut
  .handler
  .fa {
  transform: rotate(90deg);
}
