interface TProcessAttributes {
  id: number; // 28189
  database: string; // 'uvek-2022'
  location: string; // 'HU'
  name: string; // 'disposal, lignite ash, 0% water, to opencast refill'
  product: string; // 'disposal, lignite ash, 0% water, to opencast refill'
  unit: string; // 'kilogram'
  url: string; // '/process/28189'
}
