interface TRecentProcessesListInitParams {
  handlers: TSharedHandlers;
  events: SimpleEvents;
  recentProcessesListRender: RecentProcessesListRender;
  recentProcessesListNodes: RecentProcessesListNodes;
  // recentProcessesListHandlers: RecentProcessesListHandlers;
  // recentProcessesListStates: RecentProcessesListStates;
  // recentProcessesListPrepare: RecentProcessesListPrepare;
}
